-------------------------------------------------------------------------------
-- Warlockbox demon and demon empowerment tracker by Zaperox
-- Based on ElvUI Wild Imps Tracker By Lockslap which in turn is
-- Based on Imps by Kuni!
-------------------------------------------------------------------------------
local demonTime, demonCast, demonEmpowered = {}, {}, {}
local alreadyRegistered = false
local demonCount = 0
local empowered_demonCount = 0
local lastEnpowermentCast = 0
local petActive = 0
local petEmpowered = 0
local playerGUID
local CurrentDemon = "Succubus"

local WarlockboxGUI = CreateFrame("Frame", "WarlockboxGUI", UIParent)
WarlockboxGUI:SetBackdrop({
	bgFile = "Interface\\dialogframe\\ui-dialogbox-background-dark",
	edgeFile = "Interface\\tooltips\\UI-tooltip-Border",
	tile = true,
	tileSize = 32,
	edgeSize = 8,
	insets = {
		left = 1,
		right = 1,
		top = 1,
		bottom = 1,
	},
})
WarlockboxGUI:SetWidth(120)
WarlockboxGUI:SetHeight(30)
WarlockboxGUI:SetPoint("CENTER")

-- imp artwork
local WarlockboxGUIImp = WarlockboxGUI:CreateTexture("impGraphic")
WarlockboxGUIImp:SetTexture("Interface\\AddOns\\Warlockbox\\"..CurrentDemon..".tga")
WarlockboxGUIImp:SetWidth(75)
WarlockboxGUIImp:SetHeight(75)
WarlockboxGUIImp:SetPoint("CENTER", -40, 0)

-- Empowerment artwork
local WarlockboxGUIEmpowerment = WarlockboxGUI:CreateTexture("impGraphic")
WarlockboxGUIEmpowerment:SetTexture("Interface\\Icons\\spell_warlock_demonicempowerment")
WarlockboxGUIEmpowerment:SetWidth(30)
WarlockboxGUIEmpowerment:SetHeight(30)
WarlockboxGUIEmpowerment:SetPoint("CENTER", 75, 0)

-- count string
local demonCounter = WarlockboxGUI:CreateFontString("demonCounter")
demonCounter:SetFont("Fonts\\FRIZQT__.TTF", 24, "OUTLINE")
demonCounter:SetTextColor(1, 1, 1, 1)
demonCounter:SetText("0 / 0")
demonCounter:SetJustifyH("CENTER")
demonCounter:SetJustifyV("TOP")
demonCounter:SetPoint("CENTER", WarlockboxGUI, 25, 0)

-- frame movement
WarlockboxGUI:EnableMouse(true)
WarlockboxGUI:SetMovable(true)
WarlockboxGUI:RegisterForDrag("LeftButton")
WarlockboxGUI:SetScript("OnDragStart", function(self) self:StartMoving() end)
WarlockboxGUI:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)

local function isDemonology()
	local _, class = UnitClass("player")
	return (class == "WARLOCK" and GetSpecialization() == 2) and true or false
end

-- events
function WarlockboxGUI:CHARACTER_POINTS_CHANGED(self, event, ...)
	if isDemonology() then
		if not alreadyRegistered then
			WarlockboxGUI:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			alreadyRegistered = true
		end
		WarlockboxGUI:Show()
	else
		if alreadyRegistered then
			WarlockboxGUI:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			alreadyRegistered = false
		end
		WarlockboxGUI:Hide()
	end
end

function WarlockboxGUI:PLAYER_TALENT_UPDATE(self, event, ...)
	if isDemonology() then
		if not alreadyRegistered then
			WarlockboxGUI:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			alreadyRegistered = true
		end
		WarlockboxGUI:Show()
	else
		if alreadyRegistered then
			WarlockboxGUI:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			alreadyRegistered = false
		end
		WarlockboxGUI:Hide()
	end
end

function WarlockboxGUI:ACTIVE_TALENT_GROUP_CHANGED(self, event, ...)
	if isDemonology() then
		if not alreadyRegistered then
			WarlockboxGUI:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			alreadyRegistered = true
		end
		WarlockboxGUI:Show()
	else
		if alreadyRegistered then
			WarlockboxGUI:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			alreadyRegistered = false
		end
		WarlockboxGUI:Hide()
	end
end

function WarlockboxGUI:PLAYER_ENTERING_WORLD(self, event, ...)
	playerGUID = UnitGUID("player")
	
	-- only if they pass the checks will we actually look at the combat log
	if isDemonology() then
		WarlockboxGUI:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		WarlockboxGUI:Show()
		
		alreadyRegistered = true
	else
		WarlockboxGUI:Hide()
	end
	
	-- events to watch to see if they switched to a demo spec
	WarlockboxGUI:RegisterEvent("CHARACTER_POINTS_CHANGED")
	WarlockboxGUI:RegisterEvent("PLAYER_TALENT_UPDATE")
	WarlockboxGUI:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	
	WarlockboxGUI:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function WarlockboxGUI:COMBAT_LOG_EVENT_UNFILTERED(self, event, ...)
	local compTime = GetTime()
	local combatEvent = select(1, ...)
	local sourceGUID = select(3, ...)
	local sourceName = select(4, ...)
	local destGUID = select(7, ...)
	local destName = select(8, ...)
    local spellId = select(11, ...)
    local spellName = select(12, ...)
		
	-- time out any demons
	for index, value in pairs(demonTime) do
		if (value) < compTime then
			demonTime[index] = nil
			demonCast[index] = nil
			demonCount = demonCount - 1
            
			--print(("Demon timed out. Count: |cff00ff00%d|r"):format(demonCount))
		end
	end
	 
	-- demon died
	if combatEvent == "UNIT_DIED" then
		for index, value in pairs(demonTime) do
			if destGUID == index then
				demonTime[index] = nil
				demonCast[index] = nil
				demonCount = demonCount - 1
                
				--print(("Demon died. Count: |cff00ff00%d|r"):format(demonCount))
			end
		end
	end
	
	--[[ imp died from casting
	if combatEvent == "SPELL_CAST_SUCCESS" and sourceName == "Wild Imp" then
		for index,  value in pairs(demonCast) do
			if sourceGUID == index then
				-- remove cast
				demonCast[index] = demonCast[index] - 1
				
				-- wild imp has casted 10 times so it dies
				if demonCast[index] == 0 then
					demonCast[index] = nil
					demonTime[index] = nil
					demonCount = demonCount - 1
					
					--print(("Imp casted 10 times and died. Count: |cff00ff00%d|r"):format(demonCount))
				end
			end
		end
	end]]--
    
    function doEmpoweredDemonCount()
        empowered_demonCount = 0
        for index, value in pairs(demonTime) do
            if(demonEmpowered[index] ~= nil) then
                if(demonEmpowered[index] < GetTime() + 10) then
                    empowered_demonCount = empowered_demonCount + 1
                end
            end
        end
    end
    
    doEmpoweredDemonCount()
    
    -- player casts Demonic Empowerment
	if combatEvent == "SPELL_CAST_SUCCESS" and sourceGUID == UnitGUID("player") and spellId == 193396 then
        lastEnpowermentCast = GetTime()
        -- empower demons
        for index, value in pairs(demonTime) do
            demonEmpowered[index] = GetTime()
            --print("Demon empowered.")
        end
        doEmpoweredDemonCount()
	end
    
    -- unempower demons after 10 seconds
    for index, value in pairs(demonTime) do
        if(demonEmpowered[index] ~= nil) then --if not nil
            if(demonEmpowered[index] < GetTime() - 10) then
                demonEmpowered[index] = nil
                --print("Demon unempowered.")
            end
        end
    end
	
	-- imp summoned
	if combatEvent == "SPELL_SUMMON" and destName == "Wild Imp" and sourceGUID == playerGUID then
		demonTime[destGUID] = compTime + 12 --Imps last 12 seconds
		demonCast[destGUID] = 10	-- each imp has 10 casts
        demonEmpowered[destGUID] = nil
		demonCount = demonCount + 1
		
		--print(("Imp spawned. Count: |cff00ff00%d|r"):format(demonCount))
	end
    
    -- Dreadstalker summoned
	if combatEvent == "SPELL_SUMMON" and destName == "Dreadstalker" and sourceGUID == playerGUID then
		demonTime[destGUID] = compTime + 12 --Dreadstalkers last 12 seconds
		demonCast[destGUID] = nil --Dreadstalkers don't cast anything
        demonEmpowered[destGUID] = nil
		demonCount = demonCount + 1
        
		--print(("Dreadstalker spawned. Count: |cff00ff00%d|r"):format(demonCount))
	end
    
    -- Doomguard summoned
	if combatEvent == "SPELL_SUMMON" and destName == "Doomguard" and sourceGUID == playerGUID and not IsSpellKnown(152107) then
		demonTime[destGUID] = compTime + 25 --Doomguard last 25 seconds
		demonCast[destGUID] = nil --Doomguard don't have a limit to casts
        demonEmpowered[destGUID] = nil
		demonCount = demonCount + 1
        
		--print(("Doomguard spawned. Count: |cff00ff00%d|r"):format(demonCount))
	end
    
    -- Infernal summoned
	if combatEvent == "SPELL_SUMMON" and destName == "Infernal" and sourceGUID == playerGUID and not IsSpellKnown(152107) then
		demonTime[destGUID] = compTime + 25 --Infernal last 25 seconds
		demonCast[destGUID] = nil --Infernal don't have a limit to casts
        demonEmpowered[destGUID] = nil
		demonCount = demonCount + 1
        
		--print(("Infernal spawned. Count: |cff00ff00%d|r"):format(demonCount))
	end
    
    -- Darkglare summoned
	if combatEvent == "SPELL_SUMMON" and destName == "Darkglare" and sourceGUID == playerGUID then
		demonTime[destGUID] = compTime + 12 --Darkglare last 12 seconds
		demonCast[destGUID] = nil --Darkglare don't have a limit to casts
        demonEmpowered[destGUID] = nil
		demonCount = demonCount + 1
        
		--print(("Darkglare spawned. Count: |cff00ff00%d|r"):format(demonCount))
	end
    
    -- Grimoire of Service summon
	if combatEvent == "SPELL_SUMMON" and sourceGUID == playerGUID then
		if(strfind(spellName, "Grimoire:") ~= nil) then --if it is a grimoire summon
            demonTime[destGUID] = compTime + 25 --Grimoire of Service summon last 25 seconds
            demonCast[destGUID] = nil --don't have a limit to casts
            demonEmpowered[destGUID] = nil
            demonCount = demonCount + 1
            --print(("Grimoire of Service summon. Count: |cff00ff00%d|r"):format(demonCount))
        end
	end
    
    -- if the warlock has a pet
    if(UnitExists("pet")) then
        petActive = 1
        CurrentDemon = UnitCreatureFamily("pet")
        if(CurrentDemon ~= nil) then --if its a Enslaved demon
            WarlockboxGUIImp:SetTexture("Interface\\AddOns\\Warlockbox\\"..CurrentDemon..".tga")  
        end
        
        if(UnitBuff("pet", "Demonic Empowerment") ~= nil) then
            petEmpowered = 1
        else 
            petEmpowered = 0
        end
    else
        petActive = 0
        petEmpowered = 0
    end
	
	demonCounter:SetText((demonCount + petActive.." / |cff00ff00%d|r"):format(empowered_demonCount + petEmpowered))
end

WarlockboxGUI:SetScript("OnEvent", function(self, event, ...)
	self[event](self, event, ...)
end)
WarlockboxGUI:RegisterEvent("PLAYER_ENTERING_WORLD")